;------------------------------------------------------------------------------
; Boot sector code for Quark OS
;------------------------------------------------------------------------------

[ORG 0x7C00]
[BITS 16]

; Setup Segment Defenitions
	cli			; Disable Interrupts while changing segements
	mov ax,0x9000
	mov ss,ax
	mov sp,0x1FFF
	mov bp,0x0000		; Stack has been setup for CALL and PUSH
	sti

; Clear Screen , Show Prompt , Wait For keypress
	mov ax,0x03
	int 0x10		; Bios Video Function 3 = Clear Screen
	mov si,strPrompt
	call PrintString	; Boot Prompt
	mov ah,0x00
	int 0x16		; Wait for Keypress 

; Load the kernel from drive 0 = FDD
LoadKernel:
	xor ax,ax
	mov dl,0x00
	int 0x13	; Reset Controller

	mov bx,0x0200
	mov es,bx
	xor bx,bx  	; Destination = ES:BX = 0x0200:0x0000
	mov ah,0x02	; Service ID = read
	mov dl,0x00	; Floppy Disk = 0
	mov dh,0x00	; Head = 0
	mov ch,0x00 	; Track = 0
	mov cl,0x02 	; Sector = 2
	mov al,0x02 	; Number of sectors = 2  ( 2nd and 3rd )
	clc
	int 0x13	; Read 2 Sectors of II Stage
	jc LoadKernel	; Repeat Read if carry is set
	jmp 0x0200:0x0000

PrintString:
	cld
	mov ah,0x0E
    .PrintLoop:
	lodsb
	cmp al,0x00
	je .PrintExit
	int 0x10
	jmp .PrintLoop
    .PrintExit:
	ret

; DATA
	strPrompt db 'boot (quark):',0x00

; Padding and boot signature
times ( 510 - ($-$$) ) db 0x00
dw 0xAA55
