;------------------------------------------------------------------------------
; Kernel Code For Quark OS
;------------------------------------------------------------------------------

[ORG 0x7000]
[BITS 32]
; -----------------------------------------------------------------------------
; The Quark OS Kernel Entry
; -----------------------------------------------------------------------------
	PG1 equ 0x1007
	mov ax,0x10	; DS=ES=SS= Data Selector (Base 0)
	mov ds,ax
	mov es,ax
	mov ss,ax
	mov esp,0x0000FFFF	; Stack size = 64 Kb
				; This is the TOTAL kernel Stack.Take Care !!!!
	mov esi,kernel_message
	call PrintString

	mov ecx,1024*5
	mov eax,0x00
	mov edi,0x00
	cld 
	rep stosd

	mov eax,0x00
	mov dword [eax],PG1
	add eax,0x04
	mov dword [eax],PG1+0x1000
	add eax,0x04
	mov dword [eax],PG1+0x2000
	add eax,0x04
	mov dword [eax],PG1+0x3000
	mov edi,0x1000
	mov eax,0x0007
	cld

LoadPTE:
	stosd
	add eax,0x1000
	cmp eax,0x1000007
	jnz LoadPTE

	mov eax,0x00
	mov cr3,eax

	mov eax,cr0
	or eax,0x80000000
	mov cr0,eax

	mov esi,c_kernel_mesg
	call PrintString

	push dword ReturnFromMain	; Save return address on stack

	jmp 0x8:0x7400		; Jump to the C Kernel
				; C Kernel is at Asm Kernel + 1024 = 
				; 0x7000 + 0x0400 =  0x7400

ReturnFromMain:
	; We should actually never return from the C code
	; Handle this EXTRA-ORDINARY situation

	mov byte al,'?'
	mov byte [es:0xB8A02],al

	jmp $

; -----------------------------------------------------------------------------
; Functions used by the kernel
; -----------------------------------------------------------------------------

PrintString:
	cld
    .PrintNext:
	lodsb
	cmp al,0x00
	jz .PrintFinish
	mov ah,[color]
	call print_character
	jmp .PrintNext
    .PrintFinish:
	ret

print_character:
	pushad
	cmp al,0x0A
	jnz .CharNotLF
	add byte [y_pos],1
	jmp .CharPrintFinish
    .CharNotLF:
	cmp al,0x0D
	jnz .CharNotCR
	mov byte [x_pos],0x00
	jmp .CharPrintFinish
    .CharNotCR:
	mov ecx,eax
	mov ebx,0x00
	mov bl,[x_pos]
	shl bl,0x01
	mov eax,0x00
	mov al,[y_pos]
	mov edx,0xA0
	mul edx
	mov edi,0xB8000
	add edi,eax
	add edi,ebx
	mov ax,cx
	cld
	stosw
	add byte [x_pos] , 1
    .CharPrintFinish
	popad
	ret

; -----------------------------------------------------------------------------
; Data used by Kernel
; -----------------------------------------------------------------------------
	color db 0x0E
	x_pos db 0x00
	y_pos db 0x00
	kernel_message db 'This is the Quark OS asm Kernel',0x0D,0x0A,0x00
	c_kernel_mesg db 'Jumping to Code in main.cpp ...',0x0D,0x0A,0x00

; -----------------------------------------------------------------------------
; 1024 byte paddding
; -----------------------------------------------------------------------------
	kernel_pad times 1024-($-$$) db 0x00
