#ifndef QUARK_CONSOLE_H
#define QUARK_CONSOLE_H

#include <sys/types.h>
#include <asm/io.h>
#include <kernel/VGA.h>
#include <drivers/DeviceQueue.h>

#define BLINK 0x80


typedef enum console_colors_tag
	{
		BLACK=0x00,BLUE=0x01,GREEN=0x02,CYAN=0x03,
		RED=0x04,MAGENTA=0x05,BROWN=0x06,LIGHTGREY=0x07,
		DARKGREY=0x08,LIGHTBLUE=0x09,LIGHTGREEN=0x0A,LIGHTCYAN=0x0B,
		LIGHTRED=0x0C,LIGHTMAGENTA=0x0D,YELLOW=0x0E,WHITE=0x0F
	}Color;
/*
 *	Abstracts a very Generic Console. Depends on the VGA Driver For
 *	Row / Column count.
 *
 *	Can be of any rectangular area anywhere in the screen.
 */

class Console
{
private:

	uchar * vgamem;
	volatile uint screenHeight,screenWidth;
	volatile uint row,col,rowMin,colMin,rowMax,colMax;
	volatile byte attribute;

	DeviceQueue rawInputQ;

	// Zero Based Index For row , col throughout this class
	inline uint getVideoOffset(uint x,uint y)
	{
		return ( ( x * screenWidth * 2 ) + ( y * 2 ) );
	}

public:
	Console (VGA *,uint,uint,uint,uint,Color,Color);

	inline uint getRow()		
	{
		return row;
	}
	
	inline uint getColumn()
	{
		return col; 	   
	}
	
	inline void setRow(uint row)
	{
		this->row = row;
		MoveCursor(row,col);
	}
	
	inline void setColumn(uint col)
	{
		this->col = col;
		MoveCursor(row,col);
	}

	inline Color getForeColor()
	{
		return (Color)(attribute & 0x0F);
	}

	inline Color getBackColor()
	{
		return (Color) ((attribute & 0xF0) >> 0x04);
	}

	inline void setForeColor(Color fore)
	{
		attribute = ((attribute & 0xF0) | fore);
	}

	inline void setBackColor(Color back)
	{
		attribute = ( ((attribute & 0x0F) | (back << 4)) );
	}

	void enqueueKeyStroke(byte);
	
	char read();
	
	void write(char);
	void write(char *);
	
	void scrollUp();
	void cls();

	void NextRow();
	void NextCol();
	void PrevRow();
	void PrevCol();

	void BackSpace();

};

#endif
