#ifndef QUARK_EXCEPTIONS_H
#define QUARK_EXCEPTIONS_H

#include <sys/types.h>
#include <debug/Debug.h>

extern Debug * activeDebug;

// Structure to hold Register values after an exception
typedef struct exception_registers_tag
{
	unsigned long cr3;
	unsigned long cr2;
	unsigned long cr0;
	
	unsigned long eax;
	unsigned long ebx;
	unsigned long ecx;
	unsigned long edx;
	
	unsigned long esp;
	unsigned long ebp;
	unsigned long edi;
	unsigned long esi;
	
	unsigned long gs;
	unsigned long fs;
	unsigned long es;
	unsigned long ss;
	unsigned long ds;
	
	unsigned long index;
	unsigned long ecode;
	
	unsigned long eip;
	unsigned long cs;
	unsigned long eflags;
}ExceptionRegisters;

typedef struct Exception_tag
{
	unsigned short int id;
	char * name;
	void (* hook) ();
}Exception;

/*
 *	Simple Dialog Class For Displaying Exceptions !
 */

class ExceptionDialog
{
private:
	byte attr;
	byte a1,a2;
	uint startRow,startCol,endRow,endCol;
	uchar * vga;

	void print(uint,uint,char *,int = 0);
public:
	ExceptionDialog(byte,byte);
	void show(ExceptionRegisters *);
};

int ExceptionHandler  (ExceptionRegisters * r);
void ExceptionHandler (ExceptionRegisters r);

// C Declarations inside the asm file for GNU portability
extern "C"{
 void DivideException();
 void DebugException();
 void NMIException();
 void BreakpointException();
 void OverflowException();
 void BoundsException();
 void InvalidOpcodeException();
 void CPNotFoundException();
 void DoubleFaultException();
 void CPSegOverrunException();
 void InvalidTSSException();
 void SegNotPresentException();
 void StackSegOverrunException();
 void GeneralProtectionException();
 void PageFaultException();
 void ReservedException();
 void CPErrorException();
 void AlignmentCheckException();
 void MachineCheckException();
}
#endif
