#ifndef QUARK_GDT_H
#define QUARK_GDT_H

#define BASE_ADDR 0x6000
#define BASE_LOW  0x6000
#define BASE_HIGH 0x0000

#define GDO 0x000C

typedef enum present_tag 
	{ ABSENT=0x00,PRESENT=0x01 } presentBit;

typedef enum desc_privelege_tag 
	{ RING0=0x00,RING1=0x01,RING2=0x02,RING3=0x03} previlegeBit;

typedef enum seg_descriptor_tag
	{ SYSTEM=0x00,CODE_DATA=0x01} sdBit;

typedef enum seg_type_tag
	{ RES_1=0x00, TSS_16A=0x01, LDT=0x02, TSS_16B=0x03, 
	  CALL_GATE_16=0x04, TASK_GATE=0x05, INT_GATE_16=0x06,
	  TRAP_GATE_16=0x07, RES_2=0x08, TSS_32A=0x09, 
	  RES_3=0x0A, TSS_32B=0x0B, CALL_GATE_32=0x0C, RES_4=0x0D,
	  INT_GATE_32=0x0E, TRAP_GATE_32=0x0F
	}segmentTypeBit;

typedef struct GDT_tag
{
	unsigned short int segmentLow;
	unsigned short int baseLow;
	unsigned char baseMid;
	segmentTypeBit segmentType : 4;
	sdBit segmentDescriptor : 1;
	previlegeBit previlege : 2;
	presentBit present : 1;
	unsigned char segmentHigh:4;
	unsigned char granularity:4;
	unsigned char baseHigh;
}GlobalDescriptorTable;

typedef struct GDTR_tag
{
	unsigned short int length;
	unsigned short int low;
	unsigned short int high;
}GDTRegister;

/*
 *	Important abstraction of the protected mode GDT
 */
class GDT
{
private:
	void loadGDTRegister();
public:
	GDT();
	void initialize();
	void createEntry(GlobalDescriptorTable *,int);
};

#endif
