#include <kernel/VGA.h>
#include <kernel/Console.h>
#include <lib/string.h>
#include <kernel/Delay.h>
#include <kernel/Kernel.h>

#define TABSIZE 4

Console::Console(
		VGA *v,
		uint rMin,uint rMax,
		uint cMin,uint cMax,
		Color fore,Color back
		)
{
	vgamem = (uchar * ) 0xB8000;

	screenHeight = 50;//v->getHeight();
	screenWidth  = 80;//v->getWidth();
	
	if( rMax > screenHeight) rMax = screenHeight-1;
	if( cMax > screenWidth ) cMax = screenWidth-1;
	
	this->rowMin = rMin;
	this->rowMax = rMax;
	this->colMin = cMin;
	this->colMax = cMax;
	
	setForeColor(fore);
	setBackColor(back);

	cls();
}

void Console::cls()
{
	ulong flags = 0;
	saveFlags(flags);
	disableInterrupts();
	
	uint z = 0x00;
	for( uint r = rowMin ; r <= rowMax ; r++ )
	{	
		for( uint c = colMin ; c <= colMax ; c++ )
		{
			z = getVideoOffset(r,c);
			vgamem[z] = ' ';
			vgamem[z+1] = attribute;
		}
	}

	setRow(rowMin);
	setColumn(colMin);

	restoreFlags(flags);
}

void Console::scrollUp()
{
	ulong flags = 0;
	saveFlags(flags);
	disableInterrupts();

	uint i;

	uint bytesPerRow = screenWidth * 2;
	uint bytesToCopy = (colMax-colMin+1) * 2;
	
	for( i = rowMin ; i < rowMax ; i++ )
	{
		uchar * dest = (vgamem+i*bytesPerRow)+(2*colMin);
		uchar * src  = (vgamem+(i+1)*bytesPerRow)+(2*colMin);
		memcpy(dest,src,bytesToCopy);
	}

	uchar * cbase = (vgamem + rowMax*bytesPerRow);
	for( i = colMin ; i <= colMax ;i++)
	{
		cbase[2*i] = ' ';
		cbase[2*i+1] = attribute;
	}

	restoreFlags(flags);
}

void Console::write(char ch)
{
	ulong flags = 0;
	saveFlags(flags);
	disableInterrupts();

	switch(ch)
	{
		case '\r':
		case '\n':
			NextRow();
			break;
		case '\t':
			for(int k=0;k<TABSIZE;k++)
				write(' ');
			break;
		default:
			uint z = getVideoOffset(row,col);
			vgamem[z] = ch;
			vgamem[z+1] = attribute;
			NextCol();
			break;
	}
	restoreFlags(flags);
}

void Console::write(char * str)
{
	while(*str)
		write(*str++);
}

void Console::NextRow()
{
	setColumn(colMin);
	if(row == rowMax)
		scrollUp();
	else
		setRow(++row);
}

void Console::NextCol()
{
	if(col == colMax)
		NextRow();
	else
		setColumn(++col);
}

void Console::PrevRow()
{
	if(row == rowMin)
		return;
	else
	{
		setColumn(colMax);
		setRow(--row);
	}
}

void Console::PrevCol()
{
	if(col == colMin)
		PrevRow();
	else
		setColumn(--col);
}

void Console::BackSpace()
{
	ulong flags = 0;
	saveFlags(flags);
	disableInterrupts();
	
	PrevCol();
	uint z = getVideoOffset(this->row,this->col);
	vgamem[z] = ' ';
	vgamem[z+1] = attribute;

	restoreFlags(flags);
}

void Console::enqueueKeyStroke(byte ascii)
{
	rawInputQ.addRawData(ascii);
}

char Console::read()
{
	// Poll Device Queue until a character is available
	while(rawInputQ.empty())
		delay(1);
	return rawInputQ.getRawData();
}
