#include <kernel/DMA.h>
#include <kernel/Kernel.h>
#include <asm/io.h>

#define READ  1
#define WRITE 0

// IO ADDRESS FOR THE 4 CHANNELS
const static DMAChannelInfo channelInfo[] =
{
	//PAGE,OFFSET,LENGTH
	{ 0x87, 0x00, 0x01 },
	{ 0x83, 0x02, 0x03 },
	{ 0x81, 0x04, 0x05 },
	{ 0x82, 0x06, 0x07 }
};

DMAController::DMAController()
{
}

void DMAController::initialize()
{
	outb(0x0F,0x0F);
}

void DMAController::transfer
		(byte channel , ulong address , uint length , byte read)
{
	ulong page,offset;
	ulong flags = 0;

	page = address >> 16;
	offset = address & 0xFFFF;
	length--;

	saveFlags(flags);
	disableInterrupts();

	// Hold Channel
	outb(0x0A,channel|0x04);

	// Clear Flip Flop
	outb(0x0C,0x00);

	// Set DMA Mode
	outb(0x0B,channel+(read? 0x48 : 0x44));

	// Send offset info
	outb(channelInfo[channel].offset,(byte) offset & 0x00FF);
	outb(channelInfo[channel].offset,(byte) offset >> 8);

	// Send length info
	outb(channelInfo[channel].length,(byte) length & 0x00FF);
	outb(channelInfo[channel].length,(byte) length >> 8);

	// Release Hold and start asynchronous transfer
	outb(0x0A,channel);
	
	restoreFlags(flags);
}
