#include <drivers/DeviceQueue.h>

/*
 * 	The Input Monitor Routine Repeatedly Polls the DeviceQueue.
 * 	While doing so the FRONT , REAR , SIZE variables have to be
 * 	declared volatile coz they are not modified by the polling loop
 * 	but by an external routine, namely the Keyboard IRQ Handler
 */

// Note: Volatile Function implies this is a volatile pointer.

DeviceQueue::DeviceQueue()
{
	front = rear = size = 0;
}

volatile void DeviceQueue::addRawData(byte d)
{
	if(full()) return;
	
	uint r = rear;
	rear = ++rear % QUEUE_SIZE;
	++size;
	data[r] = d;
}

volatile byte DeviceQueue::getRawData()
{
	if(empty()) return 0x00;

	uint f = front;
	front = ++front % QUEUE_SIZE;
	--size;
	return data[f];
}

volatile int DeviceQueue::empty()
{
	return (size==0);
}

volatile int DeviceQueue::full()
{
	return (size==QUEUE_SIZE);
}
