#include <kernel/Exceptions.h>
#include <kernel/Kernel.h>
#include <lib/string.h>
#include <lib/ctype.h>
#include <lib/stdio.h>
#include <kernel/Speaker.h>
#include <kernel/Delay.h>

// From the omni-present Ralf Brown's Interrupt List

Exception exceptions[32] = {

	{ 0x00 , "Zero Divide Exception",	DivideException } ,
	{ 0x01 , "Single Step Trap",		DebugException } ,
	{ 0x02 , "Non Maskable Interrupt",	NMIException } ,
	{ 0x03 , "Break Point Exception",	BreakpointException } ,
	{ 0x04 , "Overflow Exception",		OverflowException } ,
	{ 0x05 , "Bounds Check Failed",		BoundsException } ,
	{ 0x06 , "Invalid Opcode",		InvalidOpcodeException } ,
	{ 0x07 , "Coprocessor Not Found",	CPNotFoundException } ,
	{ 0x08 , "Double Fault",		DoubleFaultException } ,
	{ 0x09 , "Coprocessor Segment Overrun",	CPSegOverrunException } ,
	{ 0x0A , "Invalid TSS",			InvalidTSSException } ,
	{ 0x0B , "Segment Not Present",		SegNotPresentException } ,
	{ 0x0C , "Stack Exception",		StackSegOverrunException } ,
	{ 0x0D , "General Protection Fault",	GeneralProtectionException } ,
	{ 0x0E , "Page Fault",			PageFaultException } ,
	{ 0x0F , "Reserved",			ReservedException } ,
	{ 0x10 , "Coprocessor Error",		CPErrorException },
	{ 0x11 , "Alignment Check Error",	AlignmentCheckException } ,
	{ 0x12 , "Machine Check Error",		MachineCheckException } ,
	{ 0x13 , "Reserved Exception",		ReservedException } ,
	{ 0x14 , "Reserved Exception",		ReservedException } ,
	{ 0x15 , "Reserved Exception",		ReservedException } ,
	{ 0x16 , "Reserved Exception",		ReservedException } ,
	{ 0x17 , "Reserved Exception",		ReservedException } ,
	{ 0x18 , "Reserved Exception",		ReservedException } ,
	{ 0x19 , "Reserved Exception",		ReservedException } ,
	{ 0x1A , "Reserved Exception",		ReservedException } ,
	{ 0x1B , "Reserved Exception",		ReservedException } ,
	{ 0x1C , "Reserved Exception",		ReservedException } ,
	{ 0x1D , "Reserved Exception",		ReservedException } ,
	{ 0x1E , "Reserved Exception",		ReservedException } ,
	{ 0x1F , "Reserved Exception",		ReservedException } ,
			   };

ExceptionDialog::ExceptionDialog(byte a1,byte a2)
{
	vga = (uchar *)0xB8000;

	this->a1 = a1;
	this->a2 = a2;

	this->attr = a2;

	// 30 Rows and 60 Columns
	startRow = 10;
	startCol = 10;
	endRow = 40;
	endCol = 70;
}


void ExceptionDialog::show(ExceptionRegisters * reg)
{
	ulong flags = 0;
	saveFlags(flags);
	disableInterrupts();

	char buf[80];
	uint start;
	char * msgQuark = "Quark OS : An Unexpected Exception Occured";
	char * msgReboot = "PRESS [ CTRL + ALT + DEL ] TO REBOOT";

	for ( uint r = startRow ; r <= endRow ; r++ )
	{
		for( uint c = startCol ; c <= endCol ; c++ )
		{		
			uint z = ( r * 80 + c ) * 2;
			vga[z+0] = ' ' ;
			vga[z+1] = attr;
		}
	}

	attr = a2;
	start=((endCol-startCol)-strlen(msgQuark))/2;
	this->print(1,start,msgQuark);

	attr = a1;
	start=((endCol-startCol)-strlen(exceptions[reg->index].name))/2;
	this->print(3,start,exceptions[reg->index].name);


	attr = a2;
	sprintf(buf,"General Purpose Registers");
	start=((endCol-startCol)-strlen(buf))/2;
	this->print(5,start,buf);

	attr = a1;
	sprintf(buf," EAX = %08lX EBX = %08lX ECX = %08lX EDX = %08lX",
		reg->eax , reg->ebx , reg->ecx , reg->edx );
	start = ( (endCol-startCol)-strlen(buf) )/2;
	this->print(7,start,buf);


	attr = a2;
	sprintf(buf,"Index and Stack Registers");
	start=((endCol-startCol)-strlen(buf))/2;
	this->print(10,start,buf);

	attr = a1;
	sprintf(buf," ESP = %08lX EBP = %08lX EDI = %08lX ESI = %08lX",
		reg->esp , reg->ebp , reg->edi , reg->esi);
	start = ( (endCol-startCol)-strlen(buf) )/2;
	this->print(12,start,buf);
	

	attr = a2;
	sprintf(buf,"Segment Registers");
	start=((endCol-startCol)-strlen(buf))/2;
	this->print(15,start,buf);

	attr = a1;
	sprintf(buf," DS  = %08lX ES  = %08lX FS  = %08lX GS  = %08lX",
		reg->ds , reg->es , reg->fs , reg->gs);
	start = ( (endCol-startCol)-strlen(buf) )/2;
	this->print(17,start,buf);


	attr = a2;
	sprintf(buf,"Execution Registers");
	start=((endCol-startCol)-strlen(buf))/2;
	this->print(20,start,buf);

	attr = a1;
	sprintf(buf," CS  = %08lX SS  = %08lX EIP = %08lX EFL = %08lX",
		reg->cs , reg->ss , reg->eip , reg->eflags);
	start = ( (endCol-startCol)-strlen(buf) )/2;
	this->print(22,start,buf);


	attr = a2;
	sprintf(buf,"Control Registers");
	start=((endCol-startCol)-strlen(buf))/2;
	this->print(25,start,buf);
	
	attr = a1;
	sprintf(buf," CR0 = %08lX CR1 = %08lX CR2 = %08lX CR3 = %08lX",
		reg->cr0 , (ulong)0L , reg->cr2 , reg->cr3 );

	start = ( (endCol-startCol)-strlen(buf) )/2;
	this->print(27,start,buf);
	
	attr = a2;
	start = ( (endCol-startCol)-strlen(msgReboot) )/2;
	this->print(29,start,msgReboot,1);

	restoreFlags(flags);
}

void ExceptionDialog::print(uint row,uint col,char * str,int blink)
{
	ulong flags = 0;
	saveFlags(flags);
	disableInterrupts();

	uint z = ( ( (startRow+row) * 80 ) + (startCol+col) ) * 2;
	while(*str)
	{
		vga[z++] = *str++;
		vga[z++] = blink?attr|0x80:attr;
	}
	restoreFlags(flags);
}

	
int ExceptionHandler(ExceptionRegisters * r)
{
	activeDebug->write("Quark OS : The Red Screen of Death");
	
	ExceptionDialog dialog = ExceptionDialog(0x4E,0x4F);
	dialog.show(r);

	Speaker::sound(300);
	delay(500);
	Speaker::nosound();

hang_until_reboot:
	goto hang_until_reboot;

	return 0;
}

void KernelExceptionHandler(ExceptionRegisters r)
{
	ExceptionHandler(&r);
}
