#include <kernel/GDT.h>
/*
 *	GDT and IDT based on that of Stupid OS
 *	http://geocities.yahoo.com/StupidOS
 */

GDT::GDT()
{
}

void GDT::initialize()
{
	GlobalDescriptorTable gdt;

	// Null Descriptor
	gdt.segmentLow		=	0x00;
	gdt.baseLow		=	0x00;
	gdt.baseMid		=	0x00;
	gdt.segmentType		=	RES_1;
	gdt.segmentDescriptor	=	SYSTEM;
	gdt.previlege		=	RING0;
	gdt.present		=	ABSENT;
	gdt.segmentHigh		=	0x00;
	gdt.granularity		=	0x00;
	gdt.baseHigh		=	0x00;
	createEntry(&gdt,0);

	// Code Descriptor
	gdt.segmentLow		=	0x0FFF;	// 1 MB Mark
	gdt.baseLow		=	0x0000;
	gdt.baseMid		=	0x00;
	gdt.segmentType		=	RES_3;
	gdt.segmentDescriptor	=	CODE_DATA;
	gdt.previlege		=	RING0;
	gdt.present		=	PRESENT;
	gdt.segmentHigh		=	0x00;
	gdt.granularity		=	GDO;	// Multiply limit by 4K
	gdt.baseHigh		=	0x00;
	createEntry(&gdt,1);

	// Data Descriptor
	gdt.segmentLow		=	0x0FFF;
	gdt.baseLow		=	0x0000;
	gdt.baseMid		=	0x00;
	gdt.segmentType		=	LDT;
	gdt.segmentDescriptor	=	CODE_DATA;
	gdt.previlege		=	RING0;
	gdt.present		=	PRESENT;
	gdt.segmentHigh		=	0x00;
	gdt.granularity		=	GDO;
	gdt.baseHigh		=	0x00;
	createEntry(&gdt,2);

	// All Entries From 3 To 255 are Null
	gdt.segmentLow		=	0x00;
	gdt.baseLow		=	0x00;
	gdt.baseMid		=	0x00;
	gdt.segmentType		=	RES_1;
	gdt.segmentDescriptor	=	SYSTEM;
	gdt.previlege		=	RING0;
	gdt.present		=	ABSENT;
	gdt.segmentHigh		=	0x00;
	gdt.granularity		=	0x00;
	gdt.baseHigh		=	0x00;

	for(int i=3;i<256;i++)
		createEntry(&gdt,i);
	loadGDTRegister();

	asm("mov $0x0010 , %ax");
	asm("mov %ax,%ds");
	asm("mov %ax,%ss");
	asm("mov %ax,%es");
	asm("mov %ax,%gs");
	asm("mov %ax,%fs");
}

void GDT::createEntry( GlobalDescriptorTable * g , int n)
{
	GlobalDescriptorTable *gdt = (GlobalDescriptorTable *) BASE_ADDR + n;
	gdt->segmentLow = g->segmentLow;
	gdt->baseLow = g->baseLow;
	gdt->baseMid = g->baseMid;
	gdt->segmentType = g->segmentType;
	gdt->segmentDescriptor = g->segmentDescriptor;
	gdt->previlege = g->previlege;
	gdt->present = g->present;
	gdt->segmentHigh = g->segmentHigh;
	gdt->granularity = g->granularity;
	gdt->baseHigh = g->baseHigh;
}

void GDT::loadGDTRegister()
{
	GDTRegister gdtReg;
	gdtReg.length	= (unsigned short int) 2048;
	gdtReg.low 	= (unsigned short int) BASE_LOW;
	gdtReg.high	= (unsigned short int) BASE_HIGH;
	asm( "lgdt (%0)": :"p" (&gdtReg) );
}
