#include <drivers/Keyboard.h>
#include <kernel/Kernel.h>
#include <drivers/KeyMap.h>
#include <kernel/Shutdown.h>
#include <lib/ctype.h>

byte ctrl,alt,shift,LED;

Keyboard::Keyboard()
{
	ctrl = alt = shift = LED = 0x00;
}

void Keyboard::initialize(PIC * pic,IDT * idt)
{
	updateLED(0x00);
	idt->createEntry(
		0x21,(uint)Handler_KeyboardIRQ,
		CS_SELECTOR,INTR_GATE|BITS32|PRESENT
			);

	pic->enableIRQ(KEYBOARD);
}

int Keyboard::isUsefulKey(byte scan)
{
	return  (
			scan != 0x3A	&&	// CAPS   LOCK
			scan != 0x45	&&	// NUM    LOCK
			scan != 0x46	&&	// SCROLL LOCK
			scan != 0x2A	&&	// L SHIFT
			scan != 0x36	&&	// R SHIFT
			scan != 0x38	&&	// ALT
			scan != 0x1D	 	// CTRL
		);
}

void Keyboard::updateLED(byte led)
{
	while( inb(KEYB_CTRL) & 0x02 ) ;
	outb(KEYB_PORT,0xED);
	while( inb(KEYB_CTRL) & 0x02 ) ;
	outb(KEYB_PORT,led);
}


int Handler_KeyboardIRQ()
{
	ulong flags = 0;
	saveFlags(flags);
	disableInterrupts();

	byte scanCode,asciiCode;
	
	scanCode = inb(0x60);
	
	asciiCode = 0x00;
	if(ctrl && alt && (scanCode == 0x53))
	{
		kernelReboot();
	}
	switch(scanCode)
	{
		case CAPS_LOCK:
		{
			LED ^= LED_CAPS_LOCK;
			Keyboard::updateLED(LED);
			break;
		}
		case NUM_LOCK:
		{
			LED ^= LED_NUM_LOCK;
			Keyboard::updateLED(LED);
			break;
		}
		case SCROLL_LOCK:
		{
			LED ^= LED_SCROLL_LOCK;
			Keyboard::updateLED(LED);
			break;
		}
	}


	if ( (scanCode == 0x1D) && !ctrl)
		ctrl = 0x01;
	if ( scanCode == (0x80 + 0x1D) )
		ctrl = 0x00;


	if( (scanCode == 0x2A || scanCode == 0x36) && !shift )
		shift = 0x01;	
	if( (scanCode == 0x80+0x2A) || (scanCode == 0x80+0x36) )
		shift = 0x00;

	
	if( (scanCode == 0x38) && !alt )
		alt = 0x01;
	if( scanCode == 0x80+0x38 )
		alt = 0x00;
	
	// If Release Code And Not Press Code
	if(scanCode <= 0x53)
	{
		// Ascii Code
		asciiCode = 0x00;
		if ( shift && ( LED & LED_CAPS_LOCK ) )
			asciiCode = scan2ascii[scanCode][6];
		else if ( shift )
			asciiCode = scan2ascii[scanCode][1];
		else if ( ctrl )
			asciiCode = scan2ascii[scanCode][2];
		else if ( alt )
			asciiCode = scan2ascii[scanCode][3];
		else if (shift && ( LED & LED_NUM_LOCK ))
			asciiCode = scan2ascii[scanCode][7];
		else if (LED & LED_CAPS_LOCK)
			asciiCode = scan2ascii[scanCode][5];
		else if (LED & LED_NUM_LOCK)
			asciiCode = scan2ascii[scanCode][4];
		else
			asciiCode = scan2ascii[scanCode][0];
		
		// Put asciiCode into the Input Queue of Active Console
		if( Keyboard::isUsefulKey (scanCode))
			activeConsole->enqueueKeyStroke(asciiCode);
	}

	// Send EOI to the Programmable Interrupt Controller
	
	restoreFlags(flags);

	outb(0x20,0x20);
	outb(0xa0,0x20);
	asm("mov %ebp,%esp");
	asm("pop %ebp");
	asm("iret");
	return 0;
}
