#include <kernel/PIC.h>
#include <asm/io.h>

PIC::PIC()
{
	IRQMasterStatus = 0x00;
	IRQSlaveStatus  = 0x00;
}

void PIC::initialize()
{
	// 8259A is operated in cascaded mode with Master / Slave
	// Therefore we need ICW1 , ICW2 , ICW3 , ICW4 in order

	/* Intialization Command Words */

	// ICW1
	outb ( MASTER_PORT_A , 0x11 );
	outb ( SLAVE_PORT_A  , 0x11 );
	// ICW2
	outb ( MASTER_PORT_B , 0x20);
	outb ( SLAVE_PORT_B  , 0x28);
	// ICW3
	outb ( MASTER_PORT_B , 0x04);
	outb ( SLAVE_PORT_B  , 0x02);
	// ICW
	outb ( MASTER_PORT_B , 0x01);
	outb ( SLAVE_PORT_B  , 0x01);

	// Disable All Interrupts Except IRQ0 using the mask
	outb ( MASTER_PORT_B , 0xFF);
	outb ( SLAVE_PORT_B  , 0xFF);
	asm("sti");
}

void PIC::enableIRQ ( byte irq )
{
	if(irq > 15) return;

	if(irq < 8)
	{
		IRQMasterStatus |= ( 0x01 << irq );
		outb(MASTER_PORT_B,(byte) ~IRQMasterStatus);
	}
	else
	{
		irq = irq - 8;
		IRQSlaveStatus |= ( 0x01 << irq );
		outb(SLAVE_PORT_B,(byte) ~IRQSlaveStatus);
		IRQMasterStatus = IRQMasterStatus | 0x04;
		outb(MASTER_PORT_B,(byte) ~IRQMasterStatus);
	}
}

void PIC::disableIRQ ( byte irq )
{

	if(irq > 15) return;

	if(irq < 8)
	{
		IRQMasterStatus &= ~( 0x01 << irq );
		outb(MASTER_PORT_B,(byte) ~IRQMasterStatus);
	}
	else
	{
		irq = irq - 8;
		IRQSlaveStatus &= ~( 0x01 << irq );
		outb(SLAVE_PORT_B,(byte) ~IRQSlaveStatus);
		IRQMasterStatus &= ~0x04;
		outb(MASTER_PORT_B,(byte) ~IRQMasterStatus);
	}	
}

