#include <asm/io.h>
#include <lib/string.h>
#include <kernel/Kernel.h>
#include <kernel/IDT.h>
#include <kernel/PIT.h>
#include <kernel/PIC.h>
#include <kernel/Bios.h>

void initTimeArea()
{
	ulong flags = 0;
	saveFlags(flags);

	disableInterrupts();
	
	uchar * v = (uchar *) 0xB8000;
	uchar str[80] =  "                              Quark Operating System                           ";

	for(int i=0;i<160;i+=2)
	{
		v[i] = str[i/2];
		v[i+1] = 0x6F;
	}
	
	restoreFlags(flags);
}

PIT::PIT()
{
}

void PIT::initialize(
		PIC * pic8259A,
		IDT * idt
		)
{
	static ushort count = PIT_FREQUENCY / 60;
	outb(PIT_MODE,0x36);
	outb(PIT_COUNTER0,(byte) count & 0xFF);	// LSB
	outb(PIT_COUNTER0,(byte) count >> 8);	// MSB

	// INT 0x20h GENERATE AN IRQ 0 COZ THE 8259A HAS BEEN REMAPPED VIA ICWs
	idt->createEntry (
		0x20,(uint)&Handler_TimerIRQ,
		CS_SELECTOR,TRAP_GATE|PRESENT|RING0|BITS32
			);

	pic8259A->enableIRQ(TIMER);
	initTimeArea();
}

volatile static int i = 0;
int Handler_TimerIRQ()
{
	ulong flags = 0;
	saveFlags(flags);
	disableInterrupts();

	i = (++i) % 1500;
	if(!i) schedule();

	// Signal EOI to the 8259 A

	restoreFlags(flags);
        outb(0x20,0x20);
	outb(0xa0,0x20);
        asm("mov %ebp,%esp");
        asm("pop %ebp");
        asm("iret");
	return 0;
}

void schedule()
{
	static uchar * v = (uchar *) 0xB8000;
	int value;
	value = readBIOS(HOUR_MASK);

	v[138] = 48 + (value/10);
	v[140] = 48 + (value%10);

	value = readBIOS(MINUTE_MASK);
	v[144] = 48 + (value/10);
	v[146] = 48 + (value%10);
	
	
	value = readBIOS(SECOND_MASK);
	v[150] = 48 + (value/10);
	v[152] = 48 + (value%10);

	v[142] = ':'; v[143] = v[143] | 0x80;
	v[148] = ':'; v[149] = v[149] | 0x80;
	return;
}
