#include <kernel/VGA.h>
#include <lib/string.h>
#include <asm/io.h>

// VGA DATA

// For 80 x 50 Console only
uchar Mode_80x50 [NUM_REGS] =
{
	0x5F, 0x4F, 0x50, 0x82, 0x55, 0x81, 0xBF, 0x1F, 0x00, 0x47,
	0x06, 0x07, 0x00, 0x00, 0x01, 0x40, 0x9C, 0x8E, 0x8F, 0x28,
	0x1F, 0x96, 0xB9, 0xA3, 0xFF, 0x00, 0x01, 0x02, 0x03, 0x04,
	0x05, 0x14, 0x07, 0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E,
	0x3F, 0x0C, 0x00, 0x0F,	0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x00, 0x10, 0x0E, 0x00, 0xFF, 0x03, 0x00, 0x03, 0x00, 0x02, 0x67
};

static uchar Bitmaps_8x8[2048] = {
/* 
 * BITMAP  FONT COPIED FROM /usr/src/linux/drivers/video/font_8x8.c
 * Courtsey : Linus Torvalds and the GNU GPL
 */
	#include "vga.font"
};

FontDescriptor Font_8x8 = { 0x0000 , 0x0100 , 0x0008 , Bitmaps_8x8 };


// END OF VGA DATA


VGA::VGA()
{
	height = width = 0x00;
}

void VGA::outsw (ushort addr, ushort * data, uint count)
{
	for (; count != 0; count--)
	{
		outw (addr, *data);
		data++;								        }
}

void VGA::poke (ulong fb, ushort ind, ushort val)
{
	*((ushort *) fb + ind) = val;
}

void VGA::pokeb (ulong fb, ushort ind, uchar val)
{
	*((ushort *) fb + ind) = val;
}

void VGA::write ( uchar * regs )
{
	uint index;
	outb (VGA_CRTC_INDEX, 17);
	outb (VGA_CRTC_DATA, inb (VGA_CRTC_DATA) & 0x7F);
	outb (VGA_CRTC_INDEX, 3);
	outb (VGA_CRTC_DATA, inb (VGA_CRTC_DATA) | 0x80);
	regs[17] &= 0x7F;
	regs[3] |= 0x80;
	
	for (index = 0; index < NUM_CRTC_REGS; index++)
        {
		outb (VGA_CRTC_INDEX, index);
		outb (VGA_CRTC_DATA, *regs);
		regs++;
	}
	
	regs += (NUM_AC_REGS + NUM_GC_REGS);
	
	for (index = 0; index < NUM_SEQ_REGS; index++)
	{
		outb (VGA_SEQ_INDEX, index);
		outb (VGA_SEQ_DATA, *regs);
		regs++;
	}
	
	outb (VGA_MISC_WRITE, *regs);
}

void VGA::setFont(FontDescriptor * font)
{
	static ushort seq_set[] = {
		0x0100, 0x0402, 0x0704, 0x0300
	};
	static ushort gc_set[] = {
		0x0204, 0x0005, 0x0C06
	};
	static ushort seq_reset[] = {
		0x0100, 0x0302, 0x0304, 0x0300
	};
	static ushort gc_reset[] = {
		0x0004, 0x1005, 0x0E06
	};
	ushort temp, dst;
	uchar *src;

	outsw (VGA_SEQ_INDEX, seq_set, sizeof (seq_set) / sizeof (ushort));
	outsw (VGA_GC_INDEX, gc_set, sizeof (gc_set) / sizeof (ushort));

	src = font->bitmaps + font->first * font->height;
	dst = font->first * 32;
	for (temp = font->first; temp < font->last; temp++)
	{
	memcpy((void *)(0xb8000 + dst),(const void*)src,(size_t)font->height);
		src += font->height;
		dst += 32;
	}
	outsw (VGA_SEQ_INDEX, seq_reset,
	       sizeof (seq_reset) / sizeof (ushort));
	outsw (VGA_GC_INDEX, gc_reset, sizeof (gc_reset) / sizeof (ushort));

	outb (VGA_CRTC_INDEX, 9);
	outb (VGA_CRTC_DATA, (inb (VGA_CRTC_DATA) & 0xE0) |
	      ((font->height - 1) & 0x1F));

	outb (VGA_CRTC_INDEX, 10);
	outb (VGA_CRTC_DATA, 7);
	outb (VGA_CRTC_INDEX, 11);
	outb (VGA_CRTC_DATA, 7);
}

void VGA::initialize(byte fore,byte back)
{
	ushort temp;
	ushort char_ht = 0x08 , csr_end = 0x07 , csr_start = 0x04;
	ushort screen_wd = 80 , screen_ht = 50;

	write((uchar *)&Mode_80x50);
	setFont(&Font_8x8);
	
	height = screen_ht;
	width = screen_wd;
	
	poke (0x40, 0x004A, screen_wd);
	poke (0x40, 0x004C, screen_wd * screen_ht * 2);
	for (temp = 0x0050; temp < 0x0060; temp += 2)
		poke (0x40, temp, 0);
	temp = csr_start;
	temp <<= 8;
	temp |= csr_end;
	poke (0x40, 0x0060, temp);
	poke (0x40, 0x0084, screen_ht - 1);
	poke (0x40, 0x0085, char_ht);

	uchar * vgamem = (uchar *)0xB8000;
	int max = 2 * height * width;

	byte attr = (back << 4) | fore;
	for(int i = 0 ; i < max ; i+=2)
	{
		vgamem[i] = ' ';
		vgamem[i+1] = attr;
	}	
}
