#include <lib/string.h>
#include <lib/ctype.h>
#include <kernel/GDT.h>
#include <kernel/IDT.h>
#include <kernel/PIC.h>
#include <kernel/PIT.h>
#include <kernel/VGA.h>
#include <kernel/DMA.h>
#include <kernel/Console.h>
#include <drivers/Keyboard.h>
#include <kernel/Speaker.h>
#include <kernel/Delay.h>
#include <shell/Shell.h>
#include <debug/Debug.h>

Console * activeConsole;
Debug * activeDebug;

int main()
{
	GDT gdt = GDT();
	IDT idt = IDT();
	VGA vga = VGA();
	PIC pic8259A = PIC();
	DMAController dma8237a = DMAController();

	PIT pit8254  = PIT();
	Keyboard kbd8042 = Keyboard();

	gdt.initialize();
	idt.initialize();
	vga.initialize(WHITE,BLACK);
	pic8259A.initialize();	
	dma8237a.initialize();

	Console con = Console(&vga,2,47,2,77,WHITE,BLUE);
	activeConsole = &con;

	Debug dbg = Debug(49,0x4F);
	activeDebug = &dbg;

	pit8254.initialize(&pic8259A,&idt);
	kbd8042.initialize(&pic8259A,&idt);

	Speaker::sound(1200);
	delay(500);
	Speaker::nosound();

	while(1)
	{
		// A Pseudo fork / exec loop
		Shell sh = Shell();
		sh.execute();
	}

	return 0;
}
