#include <sys/types.h>
#include <lib/string.h>
#include <lib/ctype.h>

/* String Functions */

size_t strlen(const char * s)
{
	const char * t = s;
	while(*t++);
	return t-s-1;
}

size_t strnlen ( const char * str , size_t maxlen )
{
	uint c = 0;
	while ( (*str++) && (c < maxlen) )
		c++;
	return c;
}

char * strcpy( char * to , const char * from)
{
	register char * ret = to;
	while( (*to++ = *from++) != 0 )
		;
	return ret;
}

char * strncpy( char * to , const char * from , size_t count )
{
	register char * ret = to;
	while(count > 0)
	{
		count--;
		if( (*to++ = *from++) == '\0') break;
	}
	while(count > 0)
	{
		count--;
		*to++ = '\0';
	}
	return ret;
}

char * strcat ( char * s , const char * t)
{
	register char * ret = s;
	while(*s) s++;
	while( (*s++ = *t++) != '\0' ) ;
	return ret;
}

char * strncat ( char * dest , char * src , register size_t n )
{
	if(n!=0)
	{
		register char * d = dest;
		register const char * s = src;
		while(*d++);
		do{
			if( (*d = *s++) == '\0')
				break;
			d++;
		}while(--n);
		*d = 0;
	}
	return dest;
}

char * strchr(const char * s , int c )
{
	while(1)
	{
		if(*s == c)
			return (char *) s;
		if(*s == '\0')
			return 0;
		s++;
	}
}

char * strstr( const char * haystack , const char * needle )
{
	int hlen = strlen(haystack);
	int nlen = strlen(needle);
	while(hlen >= nlen)
	{
		if( !memcmp(haystack,needle,nlen) )
			return (char * ) haystack;
		haystack++;
		hlen--;
	}
	return 0;
}

int strcmp ( const char * s1 , const char * s2 )
{
	register unsigned int a,b;
	while ( (a = *s1++) , (b = *s2++) , a&&b)
	{
		if( a != b ) return a-b;
	}
	return a-b;
}

int strncmp ( const char * s1 , const char * s2 , size_t n)
{
	while(n-- > 0)
	{
		if( *s1 != *s2) return *s1-*s2;
		if(*s1 == '\0') return 0;
		s1++;
		s2++;
	}
	return 0;
}

int strcasecmp ( const char * s1 , const char * s2)
{
	register const uchar * us1 = (const uchar * )s1,
				* us2 = (const uchar * )s2;
	while( tolower(*us1++) == tolower(*us2++))
		if( *us1++ == '\0') return 0;
	return tolower(*us1) - tolower(*--us2);
}

int strncasecmp ( const char * s1 , const char * s2 , register size_t n)
{
	if(n != 0)
	{
		register const uchar 	* us1 = (const uchar *)s1,
					* us2 = (const uchar *)s2;
		do{
			if( tolower(*us1) != tolower(*us2++) )
				return tolower(*us1) - tolower(*--us2);
			if( *us1++ == '\0') break;
		}while(--n);

	}
	return 0;
}


/* Mem Functions */
void * memset ( void * buf , int c , size_t len )
{
	register char * t = (char*)buf;
	while(len-- > 0)
		*t++ = c;
	return buf;
}

void * memcpy ( void * dest , const void * src , size_t size)
{
	size_t i;
	for( i = 0 ; i < size ; i++ )
	{
		* ( (uchar *) dest + i ) = * ( (uchar *) src + i );
	}
	return (void *)dest;
}

void * memchr(const void * s , register uchar c,register size_t n)
{
	if(n!=0)
	{
		register const unsigned char *p=(const unsigned char *)s;
		do{
			if(*p++ == c)
				return (void *) (p-1);
		}while(--n != 0);
	}
	
	return NULL;
}

int memcmp ( const void * m1 , const void * m2 , size_t len)
{
	register const char * s1=(const char*)m1,* s2 = (const char*)m2;
	register int a , b;
	while ( len-- > 0 )
	{
		if( ( a = *s1++ ) != ( b = *s2++) )
			return (a-b);
	}
	return 0;

}
